<?php

declare(strict_types=1);

namespace FiloBlu\Refilo\Helper;

use Magento\Framework\ObjectManagerInterface;
use Magento\PageBuilder\Model\Dom\HtmlDocumentFactory;
use Pelago\Emogrifier\CssInliner;
use Symfony\Component\CssSelector\Exception\ParseException;

use function class_exists;

/**
 *
 */
class PageBuilderConverter
{
    /**
     * @var HtmlDocumentFactory | null
     */
    private $documentFactory;

    /**
     * @param ObjectManagerInterface $objectManager
     */
    public function __construct(
        ObjectManagerInterface $objectManager
    ) {
        if (class_exists(HtmlDocumentFactory::class)) {
            $this->documentFactory = $objectManager->get(HtmlDocumentFactory::class);
        }
    }

    /**
     * @param $data
     * @return array|mixed|string|string[]|null
     */
    public function convert($data)
    {
        if (!$this->documentFactory || !method_exists(\Magento\PageBuilder\Model\Dom\Element::class, 'getOriginalElement')) {
            return $data;
        }

        $document = $this->documentFactory->create(['document' => $data]);
        $body = $document->getContents();
        $styles = $document->querySelectorAll('style');

        $stylesMap = [];

        foreach ($styles as $style) {
            foreach ($this->getSytles($style->getOriginalElement()->nodeValue) as $id => $value) {
                if (isset($stylesMap[$id])) {
                    $stylesMap[$id] .= $value;
                    continue;
                }

                $stylesMap[$id] = $value;
            }
        }

        if (empty($stylesMap)) {
            return $data;
        }

        $toSearch = implode('|', array_keys($stylesMap));
        preg_match('/<body>(.*)<\/body>/s', $body, $matches);

        return preg_replace_callback(
            "/data-pb-style=\"($toSearch)\"/m",
            static function ($matches) use ($stylesMap) {
                return 'style="' . trim($stylesMap[$matches[1]], '{}') . '"';
            },
            $matches[1]
        );
    }

    /**
     * @param string $css
     * @return array
     */
    public function getSytles(string $css)
    {
        $parser = new \Sabberworm\CSS\Parser($css);
        $cssDocument = $parser->parse();

        $lookup = [];

        foreach ($cssDocument->getAllDeclarationBlocks() as $block) {

            $d = array_reduce($block->getRules(), function ($carry, $a) {
                return (string)$carry . $a;
            }, '');

            foreach ($block->getSelectors() as $selector) {
                $selectorName = $selector->getSelector();
                $output = [];
                preg_match('/#html-body\s+\[data-pb-style=(.*)\]/', $selectorName, $output);

                $selectorId = $output[1];

                if (!isset($lookup[$selectorId])) {
                    $lookup[$selectorId] = $d;
                    continue;
                }

                $lookup[$selectorId] .= $d;
            }
        }

        return $lookup;
    }
}
